/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver uvm.

 Modifiziert fr AVR-NET-IO:	RoBue
 Version:			1.5 TEST-3
 Datum:				14.02.2009

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "config.h"
#include "usart.h"
#include "networkcard/enc28j60.h"
#include "stack.h"
#include "timer.h"
#include "httpd.h"
#include "cmd.h"
#include "telnetd.h"
#include "ntp.h"
#include "base64.h"
#include "http_get.h"
#include "lcd.h"
#include "udp_lcd.h"
#include "analog.h"


volatile unsigned int variable[MAX_VAR];

// Variablen-Array 
// zum Abspeichern verschiedener Werte
// und zum Einfgen in die Webseite %VA@00 bis %VA@17
// VA4-7	-> Analogwert von PORTA4-7
// VA8		-> Position Servo fuer Webcam
// VA9		-> Manueller Betrieb ein/aus
// VA10-17	-> Schaltwerte Temperaturen
// VA18-21	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA22-25	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA26-29	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA30-33	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA34-37	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA38-41	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA42-45	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA46-49	-> Schaltzeiten ein/aus hh,mm,hh,mm
// VA50-54	-> Reserve
// VA-Ende	-> Counter
unsigned int var_array[MAX_VAR_ARRAY] = {10,50,30,0,0,0,0,0,25,1,20,20,20,20,20,20,20,20,4,0,18,0,4,0,18,0,4,0,18,0,4,0,18,0,4,0,18,0,4,0,18,0,4,0,18,0,4,0,18,0,0,0,0,0,0,0};

// RoBue:
// Die Uhrzeit wird zentral in main.c berechnet (s.u.)
// Die anderen Programmteile lesen nur noch hh,mm,ss aus.
// -> httpd.h/c, cmd.h/c'

unsigned char hh;
unsigned char mm;
unsigned char ss;

unsigned int jetzt;

unsigned int start;
unsigned int ende;
unsigned char inside;

// Das periodische Auslesen der 1-Wire-Sensoren erfolgt in main.c (s.u.)
// Die anderen Programmteile lesen nur noch "ow_array" aus.
// -> httpd.h/c, cmd.h/c 

#if USE_OW
	#include "1-wire/ds18x20.h"
	// Variable
	// Anmerkung RoBue:
	// Anzahl der Sensoren (MAXSENSORS) und ID -> config.h
	// Speicherplatz fr 1-wire Sensorwerte (mit Minuswerten)
	// und zustzlich fr die Tages-Min/Max-Werte
	// (min -> ow_array[MAXSENSORS] bis ow_array[MAXSENSORS*2-1])
	// (max -> ow_array[MAXSENSORS*2] bis ow_array[MAXSENSORS*3-1])
	int16_t ow_array[MAXSENSORS*3];
	// Speicherplatz fr die ID der Sensoren
	// Position 0-4 eig. ID, Pos. 5+6 ist die Temperatur, Pos. 7 CRC-Byte
	PROGMEM	uint8_t		DS18B20IDs[MAXSENSORS+1][OW_ROMCODE_SIZE] = {
							OW_ID_T01,	// 1. DS18B20
							OW_ID_T02,
							OW_ID_T03, 
							OW_ID_T04,
							OW_ID_T05,
							OW_ID_T06,
							OW_ID_T07,
							OW_ID_T08,					
							OW_ID_Last };	// Endmarker

	uint8_t auslesen	= 0;	// 0 -> Sensoren nicht auslesen
	uint8_t messen		= 1;	// 1 -> Sensoren zum Messen auffordern
	uint8_t minmax		= 1;	// 1 -> Zurcksetzen der Min/Max-Werte 1x/Tag
	
#endif


// Schalten ja/nein
uint8_t schalten;

//----------------------------------------------------------------------------

// Hier startet das Hauptprogramm
// ******************************

int main(void)
{  
	//Konfiguration der Ausgnge bzw. Eingnge
	//definition erfolgt in der config.h
	DDRA = OUTA;
	#if USE_SER_LCD
		DDRC = OUTC;
	#else
		DDRC = OUTC;
		#if PORTD_SCHALT
			DDRD = OUTD;
		#endif
	#endif

	// Pullups einschalten
	PORTA |= (1 << PORTA0) | (1 << PORTA1) | (1 << PORTA2) | (1 << PORTA3);
	#if USE_SHT
		PORTA |= (1 << PORTA4);
	#else
		PORTA |= (1 << PORTA4) | (1 << PORTA5) | (1 << PORTA6);
	#endif
	
    unsigned long a;

    usart_init(BAUDRATE); // setup the UART

	
	#if USE_ADC
		ADC_Init();
	#endif
	
	usart_write("\n\rSystem Ready\n\r");
    usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
	for(a=0;a<1000000;a++){asm("nop");};

	//Applikationen starten
	stack_init();
	httpd_init();
	telnetd_init();
	
	//Spielerrei mit einem LCD
	#if USE_SER_LCD
		udp_lcd_init();
		lcd_init();
		// LCD-Ausgaben:
		lcd_clear();
	//	lcd_print(0,0,"ANDY * ZAUBERBOX");
	#endif

	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	#if USE_SER_LCD
		// IP auf LCD
		lcd_print(2,0,"%1i.%1i.%1i.%1i",myip[0],myip[1],myip[2],myip[3]);
	#endif
	
	//Globale Interrupts einschalten
	sei(); 

	
	#if USE_NTP
	        ntp_init();
	        ntp_request();
	#endif //USE_NTP


	// Startwerte fr ow_array setzen
#if USE_OW
		uint8_t i = 0;
		for (i=0;i<MAXSENSORS;i++){
			ow_array[i]=OW_START;
		}
		for (i=MAXSENSORS;i<MAXSENSORS*3;i++){
			ow_array[i]=OW_MINMAX;
		}
		DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL );
		for(a=0;a<1000000;a++){asm("nop");};
		auslesen = 0;
		minmax = 1;
#endif


//Hauptschlfeife
// *************
		
	while(1)
	{

	#if USE_ADC
		ANALOG_ON;
	#endif

	eth_get_data();
		
    //Terminalcommandos auswerten
	if (usart_status.usart_ready){
	usart_write("\r\n");
		if(extract_cmd(&usart_rx_buffer[0]))
		{
			usart_write("Ready\r\n\r\n");
		}
		else
		{
			usart_write("ERROR\r\n\r\n");
		}
		usart_status.usart_ready =0;
	
	}

	// Uhrzeit bestimmen
	hh = (time/3600)%24;
	mm = (time/60)%60;
	ss = time%60;


	// Uhrzeit auf LCD ausgeben
	#if USE_SER_LCD
		lcd_print(0,0,"%2i:%2i:%2i",hh,mm,ss);
		#if LCD_ZEILE3_PORTC
		// Status/Pin von PORTC
			lcd_print (0,0,"PORTC:%i%i%i%i%i%i%i%i",((PINC&0b00000001)>>0),((PINC&0b00000010)>>1),((PINC&0b00000100)>>2),((PINC&0b00001000)>>3),((PINC&0b00010000)>>4),((PINC&0b00100000)>>5),((PINC&0b01000000)>>6),((PINC&0b10000000)>>7));
			// Schaltautomatik ein/aus
			if (var_array[9] == 1) {
				lcd_print (3,15,"A");
			}
			else {
				lcd_print (3,15,"-");
			}
		#endif
	#endif

	#if USE_OW
	// Zurcksetzen der Min/Max-Werte um 00:00 Uhr einschalten
	if (( hh == 00 )&&( mm == 00 )) {
		minmax = 1;
	}
	#endif

	// ******************************************************************
	// 1-Wire-Temperatursensoren (DS18B20) abfragen
	// ******************************************************************

#if USE_OW

		uint8_t i = 0;
		uint8_t subzero, cel, cel_frac_bits;
		uint8_t tempID[OW_ROMCODE_SIZE];

		// Messen bei ss=5,15,25,35,45,55		
		if ( ss%10 == 5 ) {

		// Messen?
		if ( messen == 1 ) {

		// Hiermit werden ALLE Sensoren zum Messen aufgefordert.
		// Aufforderung nur bestimmter Sensoren:
		// "NULL" durch "tempID" ersetzen
		
		// usart_write("Starte Messvorgang ...\r\n");		 

		DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL );
		
		// Kein Messen mehr bis ss=5,15,25,35,45,55
		messen = 0;

		// Jetzt kann ausgelesen werden
		auslesen = 1;

		}	// -> if messen
		}	// -> if ss

		// Auslesen bei ss=8,18,28,38,48,58
		if ( ss%10 == 8 ) {

		// Auslesen?
		if ( auslesen == 1 ) {

		// (erste) ID ins RAM holen
		memcpy_P(tempID,DS18B20IDs[0],OW_ROMCODE_SIZE);	
				
		while ( tempID[0] != 0 ) {
		//while ( tempID[0] == 0x10 ) {
				
			// Hiermit wird jeweils ein einzelner Sensor ausgelesen
			// und die Temperatur in ow_array abgelegt.
			// Achtung:
			// Pro Sekunde knnen max. ca. 10 Sensoren ausgelesen werden!
			if ( DS18X20_read_meas( tempID, &subzero,&cel, &cel_frac_bits) == DS18X20_OK ) {

				ow_array[i] = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);

				// Minuswerte:
					if ( subzero )
						ow_array[i] *= (-1);
				
				// min/max:
				if ( minmax == 1 ) {
					// Zurcksetzen der Min/Max_Werte 1x/Tag
					// auf die gerade aktuellen Temperaturen 
					ow_array[i+MAXSENSORS] = ow_array[i];
					ow_array[i+MAXSENSORS*2] = ow_array[i];
				}
				else {
					// Abgleich der Temp. mit den gespeicherten Min/Max-Werten
					if (ow_array[i]  < ow_array[i+MAXSENSORS])
        					ow_array[i+MAXSENSORS] = ow_array[i];
               				if (ow_array[i]  > ow_array[i+MAXSENSORS*2])
						ow_array[i+MAXSENSORS*2] = ow_array[i];
        			}

				//TWert = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);
				//ow_array[i] = TWert;
				
			}	// -> if
			else {
				usart_write("\r\nCRC Error (lost connection?) ");
				DS18X20_show_id_uart( tempID, OW_ROMCODE_SIZE );

			}	// -> else
		
		// nchste ID ins RAM holen
		memcpy_P(tempID,DS18B20IDs[++i],OW_ROMCODE_SIZE);

		}	// -> while
		
		// Temperatur auf LCD ausgeben (IP von Startausgabe (s.o.) wird berschrieben)
		#if USE_SER_LCD
			lcd_print(0,10,"A: %i",(ow_array[TEMPWERT3]/10));
			lcd_print(1,0,"                ");	

			// Ausgabe 1. Temperatur auf LCD
			lcd_print(1,0,"WZ: %i",(ow_array[TEMPWERT1]/10));
			// Ausgabe 2./3. Temperatur auf LCD
			lcd_print(1,9,"KZ: %i",(ow_array[TEMPWERT2]/10));
	
			#if LCD_ZEILE3_TEMP
				// Weitere 2 Temperaturwerte auf LCD ausgeben (4. Zeile)
				lcd_print(3,0,"T:              ");

				TWert = ow_array[TEMPWERT3];
				// Ausgabe 1. Temperatur auf LCD
				lcd_print(3,3,"%i",(TWert/10));

				TWert = ow_array[TEMPWERT4];
				// Ausgabe 2. Temperatur auf LCD
				lcd_print(3,9,"%i",(TWert/10));
			#endif

		#endif

		}	// -> if auslesen

		auslesen = 0;	// Auslesen vorlufig abschalten
		messen = 1;	// Messen wieder ermglichen
		minmax = 0;	// Min/Max-Werte vergleichen

		}	// -> if ss
		
#endif
	

	// **********************************************************
	// Schalten der Ports (PORTC) durch bestimmte Bedingungen
	// - Temperatur (1-Wire -> PORTA7) mit/ohne Lftungsautomatik
	// - digital, analog (-> PORTA0-6)
	// - Zeit
	// **********************************************************

#if USE_AUTOMATIK

	int16_t TWert;
	
	// Automatik eingeschaltet?
	if ( var_array[9] == 1 ) {

	if ( ss%10 == 1 ) {
		schalten = 1;
	}

	// Abfrage bei ss =0,30
	if ( ss%30 == 0 ) {
	if ( schalten == 1 ) {

	jetzt  = hh * 60 + mm;   										// zu testender Zeitpunkt	
	
	// PORTC0:
	// ber Temperatur: var_array[10] - Sensor0 und Fenster PORTA0
	TWert = var_array[10]*10;		
	// Prfen, ob JETZT-Zeitpunkt in der Schaltzeit liegt
	start = var_array[18] * 60 + var_array[19];        				// Startzeit des Intervalls
	ende  = var_array[20] * 60 + var_array[21];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else 
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );
	
	if (inside) {
		if ((PINA&0b00000001) == 0 ) {		// PORTA0: Fenster geschlossen?
			if ( ow_array[0] < TWert ) {
				PORTC |= (1 << PC0);  // ein  // zu kalt
			} else {
				PORTC &= ~(1 << PC0); // aus  // warm genug	
			} 
		} else {
			PORTC &= ~(1 << PC0); // aus  // Fenster offen
		}	
	} else {  
			PORTC &= ~(1 << PC0); // aus  // ausserhalb der Schaltzeit
	}		
	
	
// PORTC1:
	// ber Temperatur: var_array[10] - Sensor1 und Fenster PORTA1
	TWert = var_array[11]*10;		
	// Prfen, ob JETZT-Zeitpunkt in der Schaltzeit liegt
	start = var_array[22] * 60 + var_array[23];        				// Startzeit des Intervalls
	ende  = var_array[24] * 60 + var_array[25];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else 
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );		
	
	if(inside) {
		if ((PINA&0b00000010) == 0 ) {		// PORTA1: Fenster geschlossen?
			if ( ow_array[1] < TWert ) {
				PORTC |= (1 << PC1);  // ein  // zu kalt
			} else {
				PORTC &= ~(1 << PC1); // aus  // warm genug	
			} 
		} else {
			PORTC &= ~(1 << PC1); // aus  // Fenster offen
		}	
	} else {  
			PORTC &= ~(1 << PC1); // aus  // ausserhalb der Schaltzeit
	}		

	
// PORTC2:
	// ber Temperatur: var_array[12] - Sensor2 und Fenster PORTA2
	TWert = var_array[12]*10;		
	// Prfen, ob JETZT-Zeitpunkt in der Schaltzeit liegt
	start = var_array[26] * 60 + var_array[27];        				// Startzeit des Intervalls
	ende  = var_array[28] * 60 + var_array[29];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );
	if(inside) {
		if ((PINA&0b00000100) == 0 ) {		// PORTA2: Fenster geschlossen?
			if ( ow_array[2] < TWert ) {
				PORTC |= (1 << PC2);  // ein  // zu kalt
			} else {
				PORTC &= ~(1 << PC2); // aus  // warm genug	
			} 
		} else {
			PORTC &= ~(1 << PC2); // aus  // Fenster offen
		}	
	} else {  
			PORTC &= ~(1 << PC2); // aus  // ausserhalb der Schaltzeit
	}	


// PORTC3:
	// ber Temperatur: var_array[13] - Sensor3 und Fenster PORTA3
	TWert = var_array[13]*10;		
	// Prfen, ob JETZT-Zeitpunkt in der Schaltzeit liegt
	start = var_array[30] * 60 + var_array[31];        				// Startzeit des Intervalls
	ende  = var_array[32] * 60 + var_array[33];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );
	if(inside) {	
		if ((PINA&0b00001000) == 0 ) {		// PORTA0: Fenster geschlossen?
			if ( ow_array[3] < TWert ) {
				PORTC |= (1 << PC3);  // ein  // zu kalt
			} else {
				PORTC &= ~(1 << PC3); // aus  // warm genug	
			} 
		} else {
			PORTC &= ~(1 << PC3); // aus  // Fenster offen
		}	
	} else {  
			PORTC &= ~(1 << PC3); // aus  // ausserhalb der Schaltzeit
	}		
	
	
// PORTC4:
	// ber Temperatur: var_array[14] - Sensor4 und Fenster PORTA4
	TWert = var_array[14]*10;		
	// Prfen, ob JETZT-Zeitpunkt in der Schaltzeit liegt
	start = var_array[34] * 60 + var_array[35];        				// Startzeit des Intervalls
	ende  = var_array[36] * 60 + var_array[37];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );
	if(inside) {	
		if ((PINA&0b00010000) == 0 ) {		// PORTA4: Fenster geschlossen?
			if ( ow_array[4] < TWert ) {
				PORTC |= (1 << PC4);  // ein  // zu kalt
			} else {
				PORTC &= ~(1 << PC4); // aus  // warm genug	
			} 
		} else {
			PORTC &= ~(1 << PC4); // aus  // Fenster offen
		}	
	} else {  
			PORTC &= ~(1 << PC4); // aus  // ausserhalb der Schaltzeit
	}		

// PORTC5:
	// ber Temperatur: var_array[15] - Sensor5 und Fenster PORTA5
	TWert = var_array[15]*10;		
	// Prfen, ob JETZT-Zeitpunkt in der Schaltzeit liegt
	start = var_array[38] * 60 + var_array[39];        				// Startzeit des Intervalls
	ende  = var_array[40] * 60 + var_array[41];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );
	if(inside) {
		if ((PINA&0b00100000) == 0 ) {		// PORTA5: Fenster geschlossen?
			if ( ow_array[5] < TWert ) {
				PORTC |= (1 << PC5);  // ein  // zu kalt
			} else {
				PORTC &= ~(1 << PC5); // aus  // warm genug	
			} 
		} else {
			PORTC &= ~(1 << PC5); // aus  // Fenster offen
		}	
	} else {  
			PORTC &= ~(1 << PC5); // aus  // ausserhalb der Schaltzeit
	}		
	
// PORTC6:
	// ber Temperatur: var_array[16] - Sensor6 und Fenster PORTA6
	TWert = var_array[16]*10;		
	// Prfen, ob JETZT-Zeitpunkt in der Schaltzeit liegt
	start = var_array[42] * 60 + var_array[43];        				// Startzeit des Intervalls
	ende  = var_array[44] * 60 + var_array[45];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );
	if(inside) {	
		if ((PINA&0b01000000) == 0 ) {		// PORTA0: Fenster geschlossen?
			if ( ow_array[6] < TWert ) {
				PORTC |= (1 << PC6);  // ein  // zu kalt
			} else {
				PORTC &= ~(1 << PC6); // aus  // warm genug	
			} 
		} else {
			PORTC &= ~(1 << PC6); // aus  // Fenster offen
		}	
	} else {  
			PORTC &= ~(1 << PC6); // aus  // ausserhalb der Schaltzeit
	}	

	
	// PORTC7 ZEITSCHALTUHR
	// ber Zeit: ein/aus
	start = var_array[46] * 60 + var_array[47];        				// Startzeit des Intervalls
	ende  = var_array[48] * 60 + var_array[49];       				// Endzeit des Intervalls
	inside = 0;
	if ( ende < start ) 
		inside = ( ( jetzt < ende ) || ( jetzt >= start ) );
	else
		inside = ( ( jetzt >= start ) && ( jetzt < ende ) );
	if(inside) {
			PORTC |= (1 << PC7); // ein
	} else {
			PORTC &= ~(1 << PC7); // aus
	}

	

	schalten = 0;	// Vorerst nicht mehr schalten

	} // -> schalten == 1
	} // -> if ss == ...
	} // -> if var_array == 1
	
#endif
	
        //Empfang von Zeitinformationen
	#if USE_NTP
		if(!ntp_timer){
		ntp_timer = NTP_REFRESH;
		ntp_request();
		}
	#endif //USE_NTP
        
           
	//USART Daten fr Telnetanwendung?
	telnetd_send_data();
    }

return(0);
}
